@@ -1,9 +1,14 @@
 import time
 import sys
+import warnings
 from math import ceil, log
 from fixate.core.common import bits, deprecated
 
 
+class MuxWarning(Warning):
+    pass
+
+
 class VirtualAddressMap:
     """
     The supervisor loops through the attached virtual multiplexers each time a mux update is triggered.
@@ -13,11 +18,39 @@ class VirtualAddressMap:
         self.address_handlers = []
         self.virtual_pin_list = []
         self._virtual_pin_values = 0b0
+        self._virtual_pin_values_active = 0b0
+        self._virtual_pin_values_clear = 0b0
+        self.mux_assigned_pins = {}
+        self._clearing_time = 0
 
     @property
     def pin_values(self):
-        return list(zip(self.virtual_pin_list,
-                        bits(self._virtual_pin_values, num_bits=len(self.virtual_pin_list), order="LSB")))
+        return list(
+            zip(
+                self.virtual_pin_list,
+                bits(
+                    self._virtual_pin_values_active,
+                    num_bits=len(self.virtual_pin_list),
+                    order="LSB",
+                ),
+            )
+        )
+
+    def active_pins(self):
+        return [
+            (
+                self.virtual_pin_list[pin],
+                self.mux_assigned_pins[self.virtual_pin_list[pin]],
+            )
+            for pin, value in enumerate(
+                bits(
+                    self._virtual_pin_values_active,
+                    num_bits=len(self.virtual_pin_list),
+                    order="LSB",
+                )
+            )
+            if value
+        ]
 
     def install_address_handler(self, handler):
         """
@@ -30,7 +63,11 @@ class VirtualAddressMap:
         # Checks
         common_elements = set(handler.pin_list).intersection(set(self.virtual_pin_list))
         if common_elements:
-            raise ValueError("Duplicate pin identifiers not allowed\n{}".format(', '.join(common_elements)))
+            raise ValueError(
+                "Duplicate pin identifiers not allowed\n{}".format(
+                    ", ".join(common_elements)
+                )
+            )
 
         self.virtual_pin_list.extend(handler.pin_list)
         self.address_handlers.append((len(self.virtual_pin_list), handler))
@@ -46,11 +83,22 @@ class VirtualAddressMap:
         """
         mux.pin_mask = []
         for itm in mux.pin_list:
+            if itm in self.mux_assigned_pins:
+                warnings.warn(
+                    "Pin {} in {} already assigned in {}".format(
+                        itm, mux, self.mux_assigned_pins[itm]
+                    ),
+                    MuxWarning,
+                )
             try:
                 mux.pin_mask.append(self.virtual_pin_list.index(itm))
             except ValueError as e:
-                raise ValueError("Multiplexer pin {} not found in Virtual Address Map".format(itm)) from e
+                raise ValueError(
+                    "Multiplexer pin {} not found in Virtual Address Map".format(itm)
+                ) from e
+            self.mux_assigned_pins[itm] = mux
         mux.update_callback = self.update_pin_values
+        mux._clear_callback = self.update_clearing_pin_values
 
     def update_defaults(self):
         """
@@ -75,6 +123,14 @@ class VirtualAddressMap:
             handler.update_output(shifted & mask)
             start_addr = addr
 
+    def update_clearing_output(self):
+        start_addr = 0x00
+        for addr, handler in self.address_handlers:
+            shifted = self._virtual_pin_values_clear >> start_addr
+            mask = (1 << (addr - start_addr)) - 1
+            handler.update_output(shifted & mask)
+            start_addr = addr
+
     def update_input(self):
         """
         Iterates through the address_handlers and reads the values back to update the pin values for the digital inputs
@@ -85,14 +141,16 @@ class VirtualAddressMap:
             values = handler.update_input()
             if values is not None:  # Handler can return valid input values
                 pin_values = []
-                for index, b in enumerate(bits(values, num_bits=len(handler.pin_list), order="LSB")):
+                for index, b in enumerate(
+                    bits(values, num_bits=len(handler.pin_list), order="LSB")
+                ):
                     pin_values.append((index + start_addr, b))
                 self.update_pin_values(pin_values, trigger_update=False)
             start_addr = addr
 
     def update_pin_values(self, values, trigger_update=True):
         """
-        :param values is a list of (index, value) tuples
+        :param values: is a list of (index, value) tuples
         Takes the values list and sets the individual bits in _virtual_pin_values that correspond to the index, value
         pairs. Then calls the update_output function.
         """
@@ -105,14 +163,51 @@ class VirtualAddressMap:
                 # Set bit
                 self._virtual_pin_values |= mask
         if trigger_update:
-            self.update_output()
+            if self._virtual_pin_values_active == self._virtual_pin_values:
+                pass  # No change in pins
+            elif self._virtual_pin_values_clear == self._virtual_pin_values_active:
+                # No clearing values required
+                self.update_output()
+            else:
+                # Do clearing output before desired signals
+                self.update_clearing_output()
+                self._virtual_pin_values_active = self._virtual_pin_values_clear
+                time.sleep(self._clearing_time)
+                self.update_output()
+            # As a trigger has occurred reset our values to match the virtual values and clearing time back to 0
+            self._clearing_time = 0
+            self._virtual_pin_values_active = self._virtual_pin_values
+            self._virtual_pin_values_clear = self._virtual_pin_values
+
+    def update_clearing_pin_values(self, values, clearing_time):
+        """
+        :param values: is a list of (index, value) tuples
+        Takes the values list and sets the individual bits in _virtual_pin_values_clear that correspond to the
+        index, value pairs. These values are then called using the logic
+        1. Update clearing pin values
+        2. Sleep for clearing time
+        3. Update the new output
+        This can be used for break before make logic in individual multiplexers
+        :return:
+        """
+        self._clearing_time = max(self._clearing_time, clearing_time)
+        for index, value in values:
+            # Get the mask
+            mask = 1 << index
+            # Clear bit
+            self._virtual_pin_values_clear &= ~mask
+            if value:
+                # Set bit
+                self._virtual_pin_values_clear |= mask
 
     def update_pin_by_name(self, name, value, trigger_update=True):
         index = self.virtual_pin_list.index(name)
         self.update_pin_values([(index, value)], trigger_update)
 
     def update_pins_by_name(self, pins, trigger_update=True):
-        pin_values = [(self.virtual_pin_list.index(name), value) for name, value in pins]
+        pin_values = [
+            (self.virtual_pin_list.index(name), value) for name, value in pins
+        ]
         self.update_pin_values(pin_values, trigger_update)
 
     def __getitem__(self, item):
@@ -129,6 +224,7 @@ class AddressHandler:
     :param pin_list: Iterable of pins (type string) that the AddressHandler handles
     :param defaults: Iterable of pins (type string subset of pin_list) that should default to high logic on reset
     """
+
     pin_list = ()
     pin_defaults = ()
 
@@ -150,6 +246,7 @@ class VirtualMux:
     default_signal = ""
     state = ""
     state_update_time = time.time()
+    clearing_time = 0
 
     def __call__(self, signal_output, trigger_update=True):
         self.multiplex(signal_output, trigger_update)
@@ -174,32 +271,78 @@ class VirtualMux:
         Converts a desired signal into an address to parse to the VirtualAddressMap
         Updates the virtual address map with the values
         """
-        values = []
         if signal_output is "":
             virtual_address = 0
         else:
             try:
                 virtual_address = self.signal_map[signal_output]
             except ValueError as e:
-                raise ValueError("signal_output {} not found in multiplexer".format(signal_output)) from e
+                raise ValueError(
+                    "signal_output {} not found in multiplexer".format(signal_output)
+                ) from e
 
-        for index, b in enumerate(bits(virtual_address, num_bits=len(self.pin_list), order="LSB")):
-            try:
-                values.append((self.pin_mask[index], b))
-            except IndexError:
-                # Should only ever happen in development
-                sys.stderr("SIGNAL OUTPUT: {}".format(signal_output))
-                sys.stderr("SIGNAL: {}".format(virtual_address))
-                sys.stderr("PIN MASK: {}".format(self.pin_mask))
-                sys.stderr("PIN LIST: {}".format(self.pin_list))
-                raise
+        values = self._build_values_update(virtual_address)
+        self.clear_callback()
         self.update_callback(values, trigger_update)
         if signal_output != self.state:
             self.state_update_time = time.time()
         self.state = signal_output
 
+    def _build_values_update(self, virtual_address):
+        values = []
+        for index, b in enumerate(
+            bits(virtual_address, num_bits=len(self.pin_list), order="LSB")
+        ):
+            try:
+                values.append((self.pin_mask[index], b))
+            except IndexError:
+                # Should only ever happen in development
+                sys.stderr("SIGNAL: {}".format(virtual_address))
+                sys.stderr("PIN MASK: {}".format(self.pin_mask))
+                sys.stderr("PIN LIST: {}".format(self.pin_list))
+                raise
+        return values
+
+    def _clear_callback(self, values, clearing_time):
+        """
+        Callback function for setting the state the mux has during the clearing of relays Overridden by the the virtual
+        address map.
+        :return:
+        """
+        raise NotImplementedError(
+            "Callback not set. Consider installing virtual mux into a virtual address map"
+        )
+
+    def clear_callback(self):
+        """
+        Callback function for setting the state the mux has during the clearing of relays
+        Override this function to set up custom behaviors during the clearing of relays.
+        For example. Setting all signals to off and then the new signal will cause the mux to act with a break before
+        make multiplexer with a clearing time that is set at the JigDriver or VirtualAddressMap level.
+        Must call to self._clear_callback to have any affect on the clearing state.
+        The clearing_time sets the minimum time that the mux stays in the clearing state before moving to the desired
+        signal state
+        Usage:
+        For setting all states to 0
+
+        def clear_callback(self):
+            values = self._build_values_update(0b0)
+            self._clear_callback(values, self.clearing_time)
+
+        For setting all states to a signal called "default"
+
+        def clear_callback(self):
+            virtual_address = self.signal_map["default"]
+            values = self._build_values_update(virtual_address)
+            self._clear_callback(values, self.clearing_time)
+
+        :return:
+        """
+
     def update_callback(self, values, trigger_update):
-        raise NotImplementedError("Callback not set. Consider installing virtual mux into a virtual address map")
+        raise NotImplementedError(
+            "Callback not set. Consider installing virtual mux into a virtual address map"
+        )
 
     def _check_duplicates(self, addr, value):
         if addr in self._reserved_addr:
@@ -208,7 +351,11 @@ class VirtualMux:
                 if addr == v:
                     dup = k
                     break
-            raise ValueError("Address 0b{:b} already in use\n{} is a duplicate of {}".format(addr, value, dup))
+            raise ValueError(
+                "Address 0b{:b} already in use\n{} is a duplicate of {}".format(
+                    addr, value, dup
+                )
+            )
         self._reserved_addr.add(addr)
 
     def map_shifted(self, base_index, start_index, values):
@@ -243,13 +390,18 @@ class VirtualMux:
                 addr |= 1 << self.pin_list.index(name)
             except ValueError as e:
                 raise Exception(
-                    'pin "{}" was not found in pin_list of VirtualMux {}'.format(name, self.__class__)) from e
+                    'pin "{}" was not found in pin_list of VirtualMux {}'.format(
+                        name, self.__class__
+                    )
+                ) from e
         self._check_duplicates(addr, value)
         self.signal_map[value] = addr
 
     def condensed_signal_map(self):
         binary_length = "0b{:0" + "{}".format(len(self.pin_list)) + "b}"
-        return sorted([(binary_length.format(ind), val) for val, ind in self.signal_map.items()])
+        return sorted(
+            [(binary_length.format(ind), val) for val, ind in self.signal_map.items()]
+        )
 
     def map_signals(self):
         """
@@ -428,6 +580,9 @@ class VirtualMux:
                 current_bits = int(ceil(log(len(branch), 2)))
                 self._map_tree(signal, current_index, base_bits + current_bits)
 
+    def __repr__(self):
+        return self.__class__.__name__
+
 
 def shift_nested(values, shift_arr):
     """
@@ -470,7 +625,8 @@ class VirtualSwitch(VirtualMux):
     the VirtualMux base class to use the callable syntax on the switch object. It remains for
     backward compatibility.
     """
-    pin_name = ''
+
+    pin_name = ""
     map_tree = ("FALSE", "TRUE")
 
     def multiplex(self, signal_output, trigger_update=True):
@@ -499,18 +655,21 @@ class VirtualSwitch(VirtualMux):
 class TestAddressHandler(AddressHandler):
     def update_output(self, value):
         print("Updating from {}".format(self.__class__.__name__))
-        print(*zip(self.pin_list, [b for b in bits(value, len(self.pin_list), order="LSB")]), sep="\n")
+        print(
+            *zip(
+                self.pin_list, [b for b in bits(value, len(self.pin_list), order="LSB")]
+            ),
+            sep="\n"
+        )
 
 
 class RelayMatrixMux(VirtualMux):
     clearing_time = 0.01
 
-    def multiplex(self, signal_output, trigger_update=True):
-        assert trigger_update, "A RelayMatrixMux will immediately update when a new signal is selected"
-        if self.state != signal_output:
-            self.update_callback([(pin, False) for pin in self.pin_mask], True)
-            time.sleep(self.clearing_time)
-        super().multiplex(signal_output, True)
+    def clear_callback(self):
+        virtual_address = self.signal_map.get(self.default_signal, 0b0)
+        values = self._build_values_update(virtual_address)
+        self._clear_callback(values, self.clearing_time)
 
 
 class JigMeta(type):
@@ -540,6 +699,7 @@ class JigDriver(metaclass=JigMeta):
 
     :attribute defaults: Iterable of the default pins to set high on driver reset
     """
+
     multiplexers = ()
     address_handlers = ()
     defaults = ()
@@ -558,6 +718,9 @@ class JigDriver(metaclass=JigMeta):
     def __getitem__(self, item):
         return self.virtual_map[item]
 
+    def active_pins(self):
+        return self.virtual_map.active_pins()
+
     def reset(self):
         """
         Reset the multiplexers to the default values
