@@ -1,7 +1,8 @@
-from abc import ABCMeta, abstractmethod
+from abc import ABCMeta
 import fixate.config
 
-def open(restrictions=None):
+
+def open():
     """Open is the public api for the daq driver for discovering and opening a connection
     to a valid daq.
     At the moment opens just instantiates the DAQmx Driver
@@ -11,9 +12,9 @@ def open(restrictions=None):
     A instantiated class connected to a valid daq
     """
     for daq, cls in fixate.config.DRIVERS["DAQ"]:
-        if daq == 'DaqMx':
+        if daq == "DaqMx":
             return cls()
 
 
 class DAQ(metaclass=ABCMeta):
-    REGEX_ID = None
\ No newline at end of file
+    REGEX_ID = None
