@@ -1,7 +1,7 @@
 import inspect
 from abc import ABCMeta, abstractmethod
 from functools import update_wrapper
-from fixate.core.discover import discover_sub_classes, open_visa_instrument
+from fixate.core.discover import open_instrument
 from fixate.core.exceptions import InstrumentFeatureUnavailable
 
 try:
@@ -12,7 +12,7 @@ except ImportError:
     number = float
 
 
-def open(restrictions=None):
+def open():
     """Open is the public api for the dmm driver for discovering and opening a connection
     to a valid Digital Multimeter.
     At the moment opens the first dmm connected
@@ -21,154 +21,221 @@ def open(restrictions=None):
     :return:
     A instantiated class connected to a valid dmm
     """
-    return open_visa_instrument("DSO", restrictions)
-
-
-def discover():
-    """Discovers the dmm classes implemented
-    :return:
-    """
-    return set(discover_sub_classes(DSO))
-
-
-def validate_specifications(_class, specifications):
-    """Validates the implemented dmm class against the specifications provided
-    :return:
-    True if all specifications are met
-    False if one or more specifications are not met by the class
-    """
-    raise NotImplementedError()
+    return open_instrument("DSO")
 
 
 class CallableNoArgs:
     def __call__(self):
-        self._call()
+        return self._call()
 
     def _call(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class CallableBool:
     def __call__(self, value: bool):
-        self._call(value)
+        return self._call(value)
 
     def _call(self, value: bool):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class SourcesCh:
     def ch1(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def ch2(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def ch3(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def ch4(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class SourcesSpecial:
     def function(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def math(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class SourcesWMem:
     def wmemory1(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def wmemory2(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class SourcesExt:
     def external(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class SourcesDig:
     def d0(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def d1(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def d2(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def d3(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def d4(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def d5(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def d6(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def d7(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def d8(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def d9(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def d10(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def d11(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def d12(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def d13(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def d14(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def d15(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
-class MeasureAllSources(SourcesCh, SourcesSpecial, SourcesWMem, SourcesDig, CallableNoArgs):
+class MeasureAllSources(
+    SourcesCh, SourcesSpecial, SourcesWMem, SourcesDig, CallableNoArgs
+):
     pass
 
 
@@ -209,35 +276,56 @@ class MultiMeasureSources(MeasureAllSources):
 class Coupling:
     def ac(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def dc(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def lf_reject(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def tv(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class Probe:
     def attenuation(self, value: number):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class VerticalUnits:
     def volts(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def amps(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class ChannelBase(CallableBool):
@@ -253,27 +341,45 @@ class ChannelBase(CallableBool):
 
     def bandwidth(self, value: number):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def bandwidth_limit(self, value: bool):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def impedance(self, value: number):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def invert(self, value: bool):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def offset(self, value: number):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def scale(self, value: number):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class Trigger:
@@ -286,39 +392,63 @@ class Trigger:
 
     def force(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def hf_reject(self, value: bool):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def hold_off(self, value: number):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def n_reject(self, value: bool):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class TrigSweep:
     def auto(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def normal(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class TrigLevel:
     def high(self, value: number):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def low(self, value: number):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class TrigMode:
@@ -333,39 +463,63 @@ class TrigEdge(CallableNoArgs):
 
     def level(self, value: number):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class TrigReject:
     def off(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def lf(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def hf(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class Slopes:
     def rising(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def falling(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def alternating(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def either(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class Acquire:
@@ -374,29 +528,47 @@ class Acquire:
 
     def normal(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def peak_detect(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def averaging(self, value: number):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def high_resolution(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class AcquireMode:
     def rtim(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def segm(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class Timebase:
@@ -405,29 +577,47 @@ class Timebase:
 
     def position(self, value: number):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def scale(self, value: number):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class TimebaseMode:
     def main(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def window(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def xy(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def roll(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class Events:
@@ -438,7 +628,10 @@ class Events:
         :return:
         """
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
 
 class MeasureInterval:
@@ -459,10 +652,51 @@ class MeasureRMS:
         self.ac = MeasureInterval()
 
 
+class Threshold:
+    def percent(self, upper: number, middle: number, lower: number):
+        """
+        :param upper: Upper Threshold
+        :param middle: Middle Threshold
+        :param lower: Lower Threshold
+        :return:
+        """
+        raise InstrumentFeatureUnavailable(
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
+
+    def absolute(self, upper: number, middle: number, lower: number):
+        """
+        :param upper: Upper Threshold
+        :param middle: Middle Threshold
+        :param lower: Lower Threshold
+        :return:
+        """
+        raise InstrumentFeatureUnavailable(
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
+
+
+class Define:
+    def __init__(self):
+        super().__init__()
+        self.threshold = Threshold()
+
+
+class Delay(CallableNoArgs):
+    def __init__(self):
+        super().__init__()
+        self.edges = MultiSlopes()
+
+
 class Measure:
     def __init__(self):
         self.counter = MeasureAllSources()
-        self.delay = MultiMeasureSources()
+        self.define = Define()
+        self.delay = Delay()
         self.duty = MeasureAllSources()
         self.fall_time = MeasureAllSources()
         self.frequency = MeasureAllSources()
@@ -485,6 +719,14 @@ class Measure:
         self.xmin = MeasureAllSources()
 
 
+class MultiSlopes(CallableNoArgs):
+    def __init__(self):
+        super().__init__()
+        self.rising = Slopes()
+        self.falling = Slopes()
+        self.alternating = Slopes()
+        self.either = Slopes()
+
 
 class DSO(metaclass=ABCMeta):
     REGEX_ID = "DSO"
@@ -499,6 +741,10 @@ class DSO(metaclass=ABCMeta):
         self.ch4 = ChannelBase("4")
         self.chmath = ChannelBase("math")
         self.chfunc = ChannelBase("func")
+        self.coupling = Coupling()
+        self.probe = Probe()
+        self.source1 = MultiMeasureSources()
+        self.source2 = MultiMeasureSources()
         self.trigger = Trigger()
         self.time_base = Timebase()
         self.acquire = Acquire()
@@ -529,17 +775,44 @@ class DSO(metaclass=ABCMeta):
     def load_setup(self, file_name):
         pass
 
+    @abstractmethod
+    def get_identity(self):
+        pass
+
     def run(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def single(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def stop(self):
         raise InstrumentFeatureUnavailable(
-            "{} not available on this device".format(inspect.currentframe().f_code.co_name))
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
+
+    def scale(self, value: number):
+        raise InstrumentFeatureUnavailable(
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
+
+    def offset(self, value: number):
+        raise InstrumentFeatureUnavailable(
+            "{} not available on this device".format(
+                inspect.currentframe().f_code.co_name
+            )
+        )
 
     def init_api(self):
         for func_str, handler, base_str in self.api:
@@ -563,8 +836,8 @@ class DSO(metaclass=ABCMeta):
             keys = [itm[0] for itm in sig.parameters.items()]
             for index, param in enumerate(nargs):
                 nkwargs[keys[index]] = param
-            new_str = base_str.format(**nkwargs)
+            # new_str = base_str.format(**nkwargs)
             # handler(self, new_str)
-            return handler(new_str)
+            return handler(base_str, **nkwargs)
 
         return update_wrapper(temp_func, func)
