@@ -7,52 +7,27 @@ from fixate.drivers.funcgen.helper import FuncGen
 
 MODES = {
     ":SINusoid": {
-        " [{frequency}]": {
-            ",[{amplitude}]": {
-                ",[{offset}]": {}}},
-        ":CH2": {
-            " [{frequency}]": {
-                ",[{amplitude}]": {
-                    ",[{offset}]": {}}}}},
+        " [{frequency}]": {",[{amplitude}]": {",[{offset}]": {}}},
+        ":CH2": {" [{frequency}]": {",[{amplitude}]": {",[{offset}]": {}}}},
+    },
     ":SQUare": {
-        " [{frequency}]": {
-            ",[{amplitude}]": {
-                ",[{offset}]": {}}},
-        ":CH2": {
-            " [{frequency}]": {
-                ",[{amplitude}]": {
-                    ",[{offset}]": {}}}}},
+        " [{frequency}]": {",[{amplitude}]": {",[{offset}]": {}}},
+        ":CH2": {" [{frequency}]": {",[{amplitude}]": {",[{offset}]": {}}}},
+    },
     ":RAMP": {
-        " [{frequency}]": {
-            ",[{amplitude}]": {
-                ",[{offset}]": {}}},
-        ":CH2": {
-            " [{frequency}]": {
-                ",[{amplitude}]": {
-                    ",[{offset}]": {}}}}},
+        " [{frequency}]": {",[{amplitude}]": {",[{offset}]": {}}},
+        ":CH2": {" [{frequency}]": {",[{amplitude}]": {",[{offset}]": {}}}},
+    },
     ":PULSE": {
-        " [{frequency}]": {
-            ",[{amplitude}]": {
-                ",[{offset}]": {}}},
-        ":CH2": {
-            " [{frequency}]": {
-                ",[{amplitude}]": {
-                    ",[{offset}]": {}}}}},
-    ":NOISe DEFault": {
-        ",[{amplitude}]": {
-            ",[{offset}]": {}}},
-    ":DC DEFault,DEFault": {
-        ",[{offset}]": {}},
-    ":USER": {
-        " [{frequency}]": {
-            ",[{amplitude}]": {
-                ",[{offset}]": {}}}}
+        " [{frequency}]": {",[{amplitude}]": {",[{offset}]": {}}},
+        ":CH2": {" [{frequency}]": {",[{amplitude}]": {",[{offset}]": {}}}},
+    },
+    ":NOISe DEFault": {",[{amplitude}]": {",[{offset}]": {}}},
+    ":DC DEFault,DEFault": {",[{offset}]": {}},
+    ":USER": {" [{frequency}]": {",[{amplitude}]": {",[{offset}]": {}}}},
 }
 
-ADV_MODES = {
-    ':SQUare:'
-    'DCYCle'
-}
+ADV_MODES = {":SQUare:" "DCYCle"}
 
 
 # -----------------------------------------------------------------------------------------------------------------------
@@ -89,37 +64,108 @@ class RigolDG1022(FuncGen):
         # Rigol Restrictions
         self.__restr_bandwidth = {"min": unit_scale("4uHz"), "max": unit_scale("20MHz")}
         self.__restr_phase = {"min": -180, "max": 180}
-        self.__restr_amplitude = {"min": unit_scale("4mVpp"), "max": unit_scale("20Vpp")}
+        self.__restr_amplitude = {
+            "min": unit_scale("4mVpp"),
+            "max": unit_scale("20Vpp"),
+        }
         self._amplitude = None
         self._store = {"ch1_duty": "50", "ch2_duty": "50"}
         self.api = [
-
             # WAVEFORM SELECTION:
             # Channel 1:
-            ("channel1.waveform.sin", self.store_and_write, ("FUNC SIN",  # base_str
-                                                             {"ch1_waveform_handler": None})),  # handler
-            ("channel1.waveform.square", self.store_and_write, ("FUNC SQU\r\nFUNC:SQU:DCYC {self._store[ch1_duty]}",
-                                                                {"ch1_waveform_handler": "channel1.waveform.square"})),
-            ("channel1.waveform.ramp", self.store_and_write, ("FUNC RAMP", {"ch1_waveform_handler": None})),
-            ("channel1.waveform.pulse", self.store_and_write, ("FUNC PULS\r\nPULS:DCYC {self._store[ch1_duty]}",
-                                                               {"ch1_waveform_handler": "channel1.waveform.pulse"})),
-            ("channel1.waveform.arb", self.store_and_write, ("FUNC USER", {"ch1_waveform_handler": None})),
-            ("channel1.waveform.triangle", self.store_and_write, ("FUNC TRI", {"ch1_waveform_handler": None})),
-            ("channel1.waveform.noise", self.store_and_write, ("FUNC NOIS", {"ch1_waveform_handler": None})),
-            ("channel1.waveform.dc", self.store_and_write, ("FUNC DC", {"ch1_waveform_handler": None})),
+            (
+                "channel1.waveform.sin",
+                self.store_and_write,
+                ("FUNC SIN", {"ch1_waveform_handler": None}),  # base_str
+            ),  # handler
+            (
+                "channel1.waveform.square",
+                self.store_and_write,
+                (
+                    "FUNC SQU\r\nFUNC:SQU:DCYC {self._store[ch1_duty]}",
+                    {"ch1_waveform_handler": "channel1.waveform.square"},
+                ),
+            ),
+            (
+                "channel1.waveform.ramp",
+                self.store_and_write,
+                ("FUNC RAMP", {"ch1_waveform_handler": None}),
+            ),
+            (
+                "channel1.waveform.pulse",
+                self.store_and_write,
+                (
+                    "FUNC PULS\r\nPULS:DCYC {self._store[ch1_duty]}",
+                    {"ch1_waveform_handler": "channel1.waveform.pulse"},
+                ),
+            ),
+            (
+                "channel1.waveform.arb",
+                self.store_and_write,
+                ("FUNC USER", {"ch1_waveform_handler": None}),
+            ),
+            (
+                "channel1.waveform.triangle",
+                self.store_and_write,
+                ("FUNC TRI", {"ch1_waveform_handler": None}),
+            ),
+            (
+                "channel1.waveform.noise",
+                self.store_and_write,
+                ("FUNC NOIS", {"ch1_waveform_handler": None}),
+            ),
+            (
+                "channel1.waveform.dc",
+                self.store_and_write,
+                ("FUNC DC", {"ch1_waveform_handler": None}),
+            ),
             # Channel 2:
-            ("channel2.waveform.sin", self.store_and_write, ("FUNC:CH2 SIN",  # base_str
-                                                             {"ch2_waveform_handler": None})),  # handler
-            ("channel2.waveform.square", self.store_and_write,
-             ("FUNC:CH2 SQU\r\nFUNC:SQU:DCYC:CH2 {self._store[ch2_duty]}",
-              {"ch2_waveform_handler": "channel2.waveform.square"})),
-            ("channel2.waveform.ramp", self.store_and_write, ("FUNC:CH2 RAMP", {"ch2_waveform_handler": None})),
-            ("channel2.waveform.pulse", self.store_and_write, ("FUNC:CH2 PULS\r\nPULS:DCYC {self._store[ch2_duty]}",
-                                                               {"ch2_waveform_handler": "channel2.waveform.pulse"})),
-            ("channel2.waveform.arb", self.store_and_write, ("FUNC:CH2 USER", {"ch2_waveform_handler": None})),
-            ("channel2.waveform.triangle", self.store_and_write, ("FUNC:CH2 TRI", {"ch2_waveform_handler": None})),
-            ("channel2.waveform.noise", self.store_and_write, ("FUNC:CH2 NOIS", {"ch2_waveform_handler": None})),
-            ("channel2.waveform.dc", self.store_and_write, ("FUNC:CH2 DC", {"ch2_waveform_handler": None})),
+            (
+                "channel2.waveform.sin",
+                self.store_and_write,
+                ("FUNC:CH2 SIN", {"ch2_waveform_handler": None}),  # base_str
+            ),  # handler
+            (
+                "channel2.waveform.square",
+                self.store_and_write,
+                (
+                    "FUNC:CH2 SQU\r\nFUNC:SQU:DCYC:CH2 {self._store[ch2_duty]}",
+                    {"ch2_waveform_handler": "channel2.waveform.square"},
+                ),
+            ),
+            (
+                "channel2.waveform.ramp",
+                self.store_and_write,
+                ("FUNC:CH2 RAMP", {"ch2_waveform_handler": None}),
+            ),
+            (
+                "channel2.waveform.pulse",
+                self.store_and_write,
+                (
+                    "FUNC:CH2 PULS\r\nPULS:DCYC {self._store[ch2_duty]}",
+                    {"ch2_waveform_handler": "channel2.waveform.pulse"},
+                ),
+            ),
+            (
+                "channel2.waveform.arb",
+                self.store_and_write,
+                ("FUNC:CH2 USER", {"ch2_waveform_handler": None}),
+            ),
+            (
+                "channel2.waveform.triangle",
+                self.store_and_write,
+                ("FUNC:CH2 TRI", {"ch2_waveform_handler": None}),
+            ),
+            (
+                "channel2.waveform.noise",
+                self.store_and_write,
+                ("FUNC:CH2 NOIS", {"ch2_waveform_handler": None}),
+            ),
+            (
+                "channel2.waveform.dc",
+                self.store_and_write,
+                ("FUNC:CH2 DC", {"ch2_waveform_handler": None}),
+            ),
             # CHANNEL CONFIGURATION:
             # Channel 1:
             ("channel1.vrms", self.write, "VOLT:UNIT VRMS\r\nVOLT {value}"),
@@ -127,7 +173,11 @@ class RigolDG1022(FuncGen):
             ("channel1.dbm", self.write, "VOLT:UNIT DBM\r\nVOLT {value}"),
             ("channel1.offset", self.write, "VOLT:OFFS {value}"),
             ("channel1.phase", self.write, "PHAS {value}"),
-            ("channel1.duty", self.store_and_execute, ({"ch1_duty": "{value}"}, "ch1_waveform_handler")),
+            (
+                "channel1.duty",
+                self.store_and_execute,
+                ({"ch1_duty": "{value}"}, "ch1_waveform_handler"),
+            ),
             ("channel1.frequency", self.write, "FREQ {value}"),
             # Channel 2:
             ("channel2.vrms", self.write, "VOLT:UNIT:CH2 VRMS\r\nVOLT {value}"),
@@ -138,8 +188,16 @@ class RigolDG1022(FuncGen):
             ("channel2.duty", self.store, {"ch2_duty": "{value}"}),
             ("channel2.frequency", self.write, "FREQ:CH2 {value}"),
             # CHANNEL ACTIVATION:
-            ("channel1._call", self.write, "OUTP {value}"),  # True won't work here needs to be ON or 1, OFF or 0
-            ("channel2._call", self.write, "OUTP:CH2 {value}"),  # True won't work here needs to be ON or 1, OFF or 0
+            (
+                "channel1._call",
+                self.write,
+                "OUTP {value}",
+            ),  # True won't work here needs to be ON or 1, OFF or 0
+            (
+                "channel2._call",
+                self.write,
+                "OUTP:CH2 {value}",
+            ),  # True won't work here needs to be ON or 1, OFF or 0
             # SYNC CONFIGURATION:
             ("sync.polarity.normal", self.write, ""),
             ("sync.mode.normal", self.write, ""),
@@ -149,7 +207,11 @@ class RigolDG1022(FuncGen):
             ("trigger.immediate", self.write, "TRIG:SOUR IMM"),
             ("trigger.external._call", self.write, "TRIG:SOUR EXT"),
             ("trigger.external.rising", self.write, "TRIG:SOUR EXT\r\n TRIG1:SLOP POS"),
-            ("trigger.external.falling", self.write, "TRIG:SOUR EXT\r\n TRIG1:SLOP NEG"),
+            (
+                "trigger.external.falling",
+                self.write,
+                "TRIG:SOUR EXT\r\n TRIG1:SLOP NEG",
+            ),
             ("trigger.manual", self.write, "TRIG:SOUR BUS"),
             ("trigger.delay", self.write, "TRIG:DEL {seconds}"),
             ("trigger.out.off", self.write, "OUTP:TRIG OFF"),
@@ -158,23 +220,61 @@ class RigolDG1022(FuncGen):
             ("trigger.out.falling", self.write, "OUTP:TRIG:SLOP NEG"),
             # Modulate
             # Channel 1:
-            ("channel1.modulate.am._call", self.store, {"ch1_modulate_state": "AM", "ch1_modulate_setting": "FREQ"}),
-            ("channel1.modulate.fm._call", self.store, {"ch1_modulate_state": "FM", "ch1_modulate_setting": "FREQ"}),
-            ("channel1.modulate.pm._call", self.store, {"ch1_modulate_state": "PM", "ch1_modulate_setting": "FREQ"}),
-            ("channel1.modulate.fsk._call", self.store, {"ch1_modulate_state": "FSK", "ch1_modulate_setting": "RATE"}),
-            ("channel1.modulate.bpsk._call", self.store, {"ch1_modulate_state": "BPSK",
-                                                          "ch1_modulate_setting": "RATE"}),
-            ("channel1.modulate.sum._call", self.store, {"ch1_modulate_state": "SUM", "ch1_modulate_setting": "RATE"}),
+            (
+                "channel1.modulate.am._call",
+                self.store,
+                {"ch1_modulate_state": "AM", "ch1_modulate_setting": "FREQ"},
+            ),
+            (
+                "channel1.modulate.fm._call",
+                self.store,
+                {"ch1_modulate_state": "FM", "ch1_modulate_setting": "FREQ"},
+            ),
+            (
+                "channel1.modulate.pm._call",
+                self.store,
+                {"ch1_modulate_state": "PM", "ch1_modulate_setting": "FREQ"},
+            ),
+            (
+                "channel1.modulate.fsk._call",
+                self.store,
+                {"ch1_modulate_state": "FSK", "ch1_modulate_setting": "RATE"},
+            ),
+            (
+                "channel1.modulate.bpsk._call",
+                self.store,
+                {"ch1_modulate_state": "BPSK", "ch1_modulate_setting": "RATE"},
+            ),
+            (
+                "channel1.modulate.sum._call",
+                self.store,
+                {"ch1_modulate_state": "SUM", "ch1_modulate_setting": "RATE"},
+            ),
             # MODULATE SOURCES:
-            ("channel1.modulate.source.internal._call",
-             self.store_and_write, ("{self._store[ch1_modulate_state]}:SOUR INT", {"ch1_modulate_source": "INT"})),
-            ("channel1.modulate.source.external",
-             self.store_and_write, ("{self._store[ch1_modulate_state]}:SOUR EXT", {"ch1_modulate_source": "EXT"})),
+            (
+                "channel1.modulate.source.internal._call",
+                self.store_and_write,
+                (
+                    "{self._store[ch1_modulate_state]}:SOUR INT",
+                    {"ch1_modulate_source": "INT"},
+                ),
+            ),
+            (
+                "channel1.modulate.source.external",
+                self.store_and_write,
+                (
+                    "{self._store[ch1_modulate_state]}:SOUR EXT",
+                    {"ch1_modulate_source": "EXT"},
+                ),
+            ),
             # MODULATE ACTIVATION:
             # Channel 1:
-            ("channel1.modulate._call", self.write,
-             "{self._store[ch1_modulate_state]}:STAT {value}\r\n{self._store[ch1_modulate_state]}:SOUR"
-             "{self._store[ch1_modulate_source]}"),
+            (
+                "channel1.modulate._call",
+                self.write,
+                "{self._store[ch1_modulate_state]}:STAT {value}\r\n{self._store[ch1_modulate_state]}:SOUR"
+                "{self._store[ch1_modulate_source]}",
+            ),
             # MODULATE OPTIONS:
             # Channel 1:
             ("channel1.modulate.am.depth", self.write, "AM:DEPT {value}"),
@@ -184,18 +284,36 @@ class RigolDG1022(FuncGen):
             ("channel1.modulate.fsk.rate", self.write, "FSK:INT:RATE {value}"),
             # MODULATE SHAPES:
             # Channel 1:
-            ("channel1.modulate.source.internal.shape.sin", self.write,
-             "{self._store[ch1_modulate_state]}:INT:FUNC SIN"),
-            ("channel1.modulate.source.internal.shape.square", self.write,
-             "{self._store[ch1_modulate_state]}:INT:FUNC SQU"),
-            ("channel1.modulate.source.internal.shape.triangle", self.write,
-             "{self._store[ch1_modulate_state]}:INT:FUNC TRI"),
-            ("channel1.modulate.source.internal.shape.up_ramp", self.write,
-             "{self._store[ch1_modulate_state]}:INT:FUNC RAMP"),
-            ("channel1.modulate.source.internal.shape.down_ramp", self.write,
-             "{self._store[ch1_modulate_state]}:INT:FUNC NRAMP"),
-            ("channel1.modulate.source.internal.shape.noise", self.write,
-             "{self._store[ch1_modulate_state]}:INT:FUNC NOIS"),
+            (
+                "channel1.modulate.source.internal.shape.sin",
+                self.write,
+                "{self._store[ch1_modulate_state]}:INT:FUNC SIN",
+            ),
+            (
+                "channel1.modulate.source.internal.shape.square",
+                self.write,
+                "{self._store[ch1_modulate_state]}:INT:FUNC SQU",
+            ),
+            (
+                "channel1.modulate.source.internal.shape.triangle",
+                self.write,
+                "{self._store[ch1_modulate_state]}:INT:FUNC TRI",
+            ),
+            (
+                "channel1.modulate.source.internal.shape.up_ramp",
+                self.write,
+                "{self._store[ch1_modulate_state]}:INT:FUNC RAMP",
+            ),
+            (
+                "channel1.modulate.source.internal.shape.down_ramp",
+                self.write,
+                "{self._store[ch1_modulate_state]}:INT:FUNC NRAMP",
+            ),
+            (
+                "channel1.modulate.source.internal.shape.noise",
+                self.write,
+                "{self._store[ch1_modulate_state]}:INT:FUNC NOIS",
+            ),
             # BURST
             # Channel 1:
             ("channel1.burst.gated._call", self.write, "BURS:MODE GAT"),
@@ -203,13 +321,20 @@ class RigolDG1022(FuncGen):
             ("channel1.burst.ncycle._call", self.write, "BURS:MODE TRIG"),
             ("channel1.burst.ncycle.cycles._call", self.write, "BURS:NCYC {cycles}"),
             ("channel1.burst.ncycle.cycles.infinite", self.write, "BURS:NCYC INF"),
-            ("channel1.burst.ncycle.burst_period", self.write, "BURS:INT:PER {seconds}"),
+            (
+                "channel1.burst.ncycle.burst_period",
+                self.write,
+                "BURS:INT:PER {seconds}",
+            ),
             ("channel1.burst.gated.positive", self.write, "BURS:GATE:POL NORM"),
             ("channel1.burst.gated.negative", self.write, "BURS:GATE:POL INV"),
             ("channel1.burst.phase", self.write, "BURS:PHAS {degrees}"),
             # Modulate Frequency
-            ("channel1.modulate.source.internal.frequency", self.write,
-             "{self._store[ch1_modulate_state]}:INT:{self._store[ch1_modulate_setting]} {value}"),
+            (
+                "channel1.modulate.source.internal.frequency",
+                self.write,
+                "{self._store[ch1_modulate_state]}:INT:{self._store[ch1_modulate_setting]} {value}",
+            ),
             # LOAD:
             # channel1:
             ("channel1.load._call", self.write, "OUTP:LOAD {ohms}"),
@@ -217,7 +342,6 @@ class RigolDG1022(FuncGen):
             # channel2:
             ("channel2.load._call", self.write, "OUTP:LOAD:CH2 {ohms}"),
             ("channel2.load.infinite", self.write, "OUTP:LOAD:CH2 INF"),
-
         ]
 
         # -----------------------------------------------------------------------------------------------------------------------
@@ -266,19 +390,19 @@ class RigolDG1022(FuncGen):
 
     @property
     def amplitude_ch1(self):
-        return self.instrument.query_ascii_values('VOLTAGE?')[0]
+        return self.instrument.query_ascii_values("VOLTAGE?")[0]
 
     @property
     def amplitude_ch2(self):
-        return self.instrument.query_ascii_values('VOLTAGE:CH2?')[0]
+        return self.instrument.query_ascii_values("VOLTAGE:CH2?")[0]
 
     @amplitude_ch1.setter
     def amplitude_ch1(self, val):
-        self._write('VOLTAGE {}'.format(val))
+        self._write("VOLTAGE {}".format(val))
 
     @amplitude_ch2.setter
     def amplitude_ch2(self, val):
-        self._write('VOLTAGE:CH2 {}'.format(val))
+        self._write("VOLTAGE:CH2 {}".format(val))
 
     @property
     def output_ch1(self):
@@ -291,7 +415,11 @@ class RigolDG1022(FuncGen):
     @output_ch1.setter
     def output_ch1(self, val):
         if val not in [True, False]:
-            raise ParameterError("Unknown output {} value for CH1\r\nPlease select True or False".format(val))
+            raise ParameterError(
+                "Unknown output {} value for CH1\r\nPlease select True or False".format(
+                    val
+                )
+            )
         if val:
             self._write("OUTP ON")
         else:
@@ -308,7 +436,11 @@ class RigolDG1022(FuncGen):
     @output_ch2.setter
     def output_ch2(self, val):
         if val not in [True, False]:
-            raise ParameterError("Unknown output {} value for CH2\nPlease select True or False".format(val))
+            raise ParameterError(
+                "Unknown output {} value for CH2\nPlease select True or False".format(
+                    val
+                )
+            )
         if val:
             self._write("OUTP:CH2 ON")
         else:
@@ -318,7 +450,11 @@ class RigolDG1022(FuncGen):
     def output_sync(self, val):
         time.sleep(0.5)
         if val not in [True, False]:
-            raise ParameterError("Unknown output {} value for SYNC\nPlease select True or False".format(val))
+            raise ParameterError(
+                "Unknown output {} value for SYNC\nPlease select True or False".format(
+                    val
+                )
+            )
         self._output_sync = val
         if self._output_sync:
             self._write("OUTP:SYNC ON")
@@ -340,11 +476,14 @@ class RigolDG1022(FuncGen):
         period. This could cause issues. Ensure that setup is in a safe state to receive such a signal.
         :return:
         """
-        self.output_ch1 = False  # Due to the 5Vpp 1kHz signal. Explicit call to turn output off first
+        # Due to the 5Vpp 1kHz signal. Explicit call to turn output off first
+        self.output_ch1 = False
         self.output_ch2 = False
         self._write("*RST")
 
-    def function(self, waveform, channel=1, duty_cycle=None, symmetry=None, phase=None, **kwargs):
+    def function(
+        self, waveform, channel=1, duty_cycle=None, symmetry=None, phase=None, **kwargs
+    ):
         """
          if parameters empty then uses previous set mode
          The mode and mode parameters are used in mode_build to search recursively through the
@@ -363,26 +502,36 @@ class RigolDG1022(FuncGen):
         if int(channel) in range(1, 3):
             channel = "CH{}".format(channel)
         else:
-            raise ValueError("Invalid channel {} use a number between 1-2".format(channel))
+            raise ValueError(
+                "Invalid channel {} use a number between 1-2".format(channel)
+            )
         mode = (waveform, channel)
         # self.reset()
         if duty_cycle is not None:
-            if waveform.upper() not in 'PULSE':
+            if waveform.upper() not in "PULSE":
                 if channel == "CH1":
-                    self._write(['FUNCtion:{}:DCYCle {}'.format(waveform.upper(), duty_cycle)])
+                    self._write(
+                        ["FUNCtion:{}:DCYCle {}".format(waveform.upper(), duty_cycle)]
+                    )
                 else:
-                    self._write(['FUNCtion:{}:DCYCle:{} {}'.format(waveform.upper(), channel, duty_cycle)])
+                    self._write(
+                        [
+                            "FUNCtion:{}:DCYCle:{} {}".format(
+                                waveform.upper(), channel, duty_cycle
+                            )
+                        ]
+                    )
             else:
                 if channel == "CH1":
-                    self._write(['PULSe:DCYC {}'.format(duty_cycle)])
+                    self._write(["PULSe:DCYC {}".format(duty_cycle)])
                 else:
-                    self._write(['PULSe:DCYC:{} {}'.format(channel, duty_cycle)])
+                    self._write(["PULSe:DCYC:{} {}".format(channel, duty_cycle)])
 
         if symmetry is not None:
             if channel == "CH1":
-                self._write(['FUNCtion:RAMP:SYMMetry {}'.format(symmetry)])
+                self._write(["FUNCtion:RAMP:SYMMetry {}".format(symmetry)])
             else:
-                self._write(['FUNCtion:RAMP:SYMMetry:{} {}'.format(channel, symmetry)])
+                self._write(["FUNCtion:RAMP:SYMMetry:{} {}".format(channel, symmetry)])
 
         if phase is not None:
             if channel == "CH1":
@@ -393,11 +542,15 @@ class RigolDG1022(FuncGen):
         self._write(["APPLy{}".format(mode_builder(MODES, {}, *mode, **kwargs))])
 
     def am(self, frequency, depth, source=None, waveform="SIN"):
-        self._write(["AM:SOURce INT",
-                     "AM:INT:FREQuency {frequency}".format(frequency=frequency),
-                     "AM:DEPTh {depth}".format(depth=depth),
-                     "AM:INT:FUNC {waveform}".format(waveform=waveform),
-                     "AM:STATe ON"])
+        self._write(
+            [
+                "AM:SOURce INT",
+                "AM:INT:FREQuency {frequency}".format(frequency=frequency),
+                "AM:DEPTh {depth}".format(depth=depth),
+                "AM:INT:FUNC {waveform}".format(waveform=waveform),
+                "AM:STATe ON",
+            ]
+        )
 
     def disable_am(self):
         self._write(["AM:STATe OFF"])
@@ -413,7 +566,7 @@ class RigolDG1022(FuncGen):
         :param mode_params:
         :return:
         """
-        raise NotImplemented()
+        raise NotImplementedError
 
     def _write(self, data):
         """
@@ -428,7 +581,7 @@ class RigolDG1022(FuncGen):
         """
         if data:
             if isinstance(data, str):
-                data = data.split('\r\n')
+                data = data.split("\r\n")
             for itm in data:
                 self.instrument.write(itm)
                 time.sleep(0.1 + len(itm) / 6000)
@@ -438,7 +591,7 @@ class RigolDG1022(FuncGen):
 
     def _check_errors(self):
         resp = self.instrument.query("SYST:ERR?")
-        code, msg = resp.strip('\n').split(',')
+        code, msg = resp.strip("\n").split(",")
         code = int(code)
         msg = msg.strip('"')
         return code, msg
@@ -455,15 +608,22 @@ class RigolDG1022(FuncGen):
             if silent:
                 return errors
             else:
-                raise InstrumentError("Error(s) Returned from FuncGen\n" +
-                                      "\n".join(["Code: {}\nMessage:{}".format(code, msg) for code, msg in errors]))
+                raise InstrumentError(
+                    "Error(s) Returned from FuncGen\n"
+                    + "\n".join(
+                        [
+                            "Code: {}\nMessage:{}".format(code, msg)
+                            for code, msg in errors
+                        ]
+                    )
+                )
 
     def write(self, base_str, *args, **kwargs):
         formatted_string = self._format_string(base_str, **kwargs)
         self._write(formatted_string)
 
     def _format_string(self, base_str, **kwargs):
-        kwargs['self'] = self
+        kwargs["self"] = self
         prev_string = base_str
         cur_string = ""
         while True:
@@ -543,3 +703,11 @@ class RigolDG1022(FuncGen):
 
         return update_wrapper(temp_func, func)
         # ------------------------------------------------------------------------------------------
+
+    def get_identity(self):
+        """
+        Query ID character string of instrument, including a field separated by 4 “,”, manufactory, model, serial number
+        and the edition number that consists of numbers and separated by “.” .
+        :return: RIGOL TECHNOLOGIES,DG1022,DG1000000002,00.01.00.04.00
+        """
+        return self.instrument.query("*IDN?").strip()
