@@ -5,4 +5,4 @@ Use lcr.open to connect to a connected digital multi meter
 Functions are dictacted by the metaclass in helper.py
 
 """
-from fixate.drivers.lcr.helper import open, discover, LCR, TestResult
\ No newline at end of file
+from fixate.drivers.lcr.helper import open, LCR, TestResult
