@@ -1,9 +1,9 @@
 from abc import ABCMeta, abstractmethod
 
-from fixate.core.discover import discover_sub_classes, open_visa_instrument
+from fixate.core.discover import open_instrument
 
 
-def open(restrictions=None):
+def open():
     """Open is the public api for the dmm driver for discovering and opening a connection
     to a valid Digital Multimeter.
     At the moment opens the first dmm connected
@@ -12,24 +12,9 @@ def open(restrictions=None):
     :return:
     A instantiated class connected to a valid dmm
     """
-    return open_visa_instrument("LCR", restrictions)
+    return open_instrument("LCR")
 
 
-def discover():
-    """Discovers the dmm classes implemented
-    :return:
-    """
-    return set(discover_sub_classes(LCR))
-
-
-def validate_specifications(_class, specifications):
-    """Validates the implemented dmm class against the specifications provided
-    :return:
-    True if all specifications are met
-    False if one or more specifications are not met by the class
-    """
-    raise NotImplementedError()
-
 class TestResult:
     Rs = None
     Cs = None
@@ -46,6 +31,7 @@ class TestResult:
     def __init__(self, **kwargs):
         self.__dict__.update(**kwargs)
 
+
 class LCR(metaclass=ABCMeta):
     REGEX_ID = "LCR"
     frequency = None
@@ -61,4 +47,8 @@ class LCR(metaclass=ABCMeta):
 
     @abstractmethod
     def reset(self):
-        pass
\ No newline at end of file
+        pass
+
+    @abstractmethod
+    def get_identity(self):
+        pass
